import editorStyle from "../util/defaultStyle";
import { getShapeName } from "../util/clazz";

export default function(G6) {
  G6.registerBehavior("dragPanelItemAddNode", {
    getDefaultCfg() {
      return {};
    },
    getEvents() {
      return {
        "canvas:mouseup": "onMouseUp",
        "canvas:mouseleave": "onMouseLeave"
      };
    },
    onMouseUp(e) {
      if (this.graph.get("onDragAddNode")) {
        let p;
        p = this.graph.getPointByClient(e.clientX, e.clientY);
        const subProcessNode = this.graph.find("node", node => {
          if (node.get("model")) {
            const { clazz } = node.get("model");
            if (clazz === "subProcess") {
              const bbox = node.getBBox();
              return p.x > bbox.minX && bbox.maxX > p.x && p.y > bbox.minY && bbox.maxY > p.y;
            }
          } else {
            return false;
          }
        });
        if (subProcessNode) {
          if (p.x > 0 && p.y > 0) {
            this._addNodeBySubProcess(p, subProcessNode);
          }
        } else if (p.x > 0 && p.y > 0) {
          this._addNode(p);
        }
      }
    },
    _addNodeBySubProcess(p, node) {
      if (this.graph.get("onDragAddNode")) {
        const addModel = this.graph.get("addModel");
        const { clazz = "userTask" } = addModel;
        addModel.shape = getShapeName(clazz);
        addModel.size =
          typeof addModel.size == "string"
            ? addModel.size.split("*")
            : addModel.size.join("*").split("*");
        const timestamp = new Date().getTime();
        const id = clazz + timestamp;
        const bbox = node.getBBox();
        const x = p.x - bbox.x - bbox.width / 2;
        const y = p.y - bbox.y - bbox.height / 2;
        const nodeCfg = {
          ...addModel,
          x,
          y,
          id
        };
        const group = node.getContainer();
        const resultModel = group.addNodeModel(node, nodeCfg);
        if (this.graph.executeCommand) {
          this.graph.executeCommand("update", {
            itemId: node.get("id"),
            updateModel: resultModel
          });
        } else {
          this.graph.updateItem(node, resultModel);
        }
      }
    },
    onMouseLeave(e) {
      if (this.graph.get("onDragAddNode")) {
        this._clearDelegate();
        this.graph.emit("afternodedragend");
      }
    },
    _clearDelegate() {
      if (this.graph.get("onDragAddNode")) {
        const delegateShape = this.graph.get("addDelegateShape");
        if (delegateShape) {
          delegateShape.remove();
          this.graph.set("addDelegateShape", null);
          this.graph.paint();
        }
      }
    },
    _addNode(p) {
      if (this.graph.get("onDragAddNode")) {
        const addModel = this.graph.get("addModel");
        const { clazz = "userTask" } = addModel;
        addModel.shape = getShapeName(clazz);
        const timestamp = new Date().getTime();
        const id = clazz + timestamp;
        const { x } = p;
        const { y } = p;
        if (this.graph.executeCommand) {
          this.graph.executeCommand("add", {
            type: "node",
            addModel: {
              ...addModel,
              x,
              y,
              id
            }
          });
        } else {
          this.graph.add("node", {
            ...addModel,
            x,
            y,
            id
          });
        }
      }
    }
  });
}
